﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml;
using System.Windows.Markup;
using System.Collections.Generic;
using System.Collections;

namespace Sample.Configurations {

    public abstract class ConfigurationBase<T> where T : new(){
        private static T instance = default(T);
        private static object sync = new object();

        private ResourceDictionary dictionary = null;

        protected ConfigurationBase() {
        }

        public static T Current {
            get {
                lock (sync) {
                    if (instance == null) {
                        instance = new T();
                    }
                    return instance;
                }
            }
        }

        protected abstract string ConfigFileName {
            get;
        }

        private void Load() {
            dictionary = LoadXaml(ConfigFileName);
        }

        protected TType GetValue<TType>(string key) {
            if (dictionary == null) {
                Load();
            }
            if (dictionary == null) {
                return default(TType);
            }
            if (dictionary.Contains(key)) {
                return (TType)dictionary[key];
            }
            return default(TType);
        }

        public IEnumerable<DictionaryEntry> All() {
            if (dictionary == null) {
                Load();
            }
            ResourceDictionary source = dictionary;
            if (source == null) {
                source = new ResourceDictionary();
            }

            foreach (DictionaryEntry entry in source) {
                yield return entry;
            }
        }

        private ResourceDictionary LoadXaml(string uri) {
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.XmlResolver = new XmlXapResolver();
            using (XmlReader reader = XmlReader.Create(uri, settings)) {
                if (reader.Read()) {
                    string xaml = reader.ReadOuterXml();
                    return XamlReader.Load(xaml) as ResourceDictionary;
                }
            }
            return null;
        }
    }
}
