﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Sample.Controls {
    public class FillSizeWrapPanel : Panel {

        public double ItemMinWidth {
            get { return (double)GetValue(ItemMinWidthProperty); }
            set { SetValue(ItemMinWidthProperty, value); }
        }

        public static readonly DependencyProperty ItemMinWidthProperty =
            DependencyProperty.Register("ItemMinWidth", typeof(double), typeof(FillSizeWrapPanel), new PropertyMetadata(double.NaN, OnItemMinWidthChanged));

        private static void OnItemMinWidthChanged(object sender, DependencyPropertyChangedEventArgs e) {
            ((FillSizeWrapPanel)sender).OnItemMinWidthChanged((double)e.NewValue, (double)e.OldValue);
        }

        private void OnItemMinWidthChanged(double newValue, double oldValue) {
            this.InvalidateMeasure();
        }

        public double ItemHeight {
            get { return (double)GetValue(ItemHeightProperty); }
            set { SetValue(ItemHeightProperty, value); }
        }

        public static readonly DependencyProperty ItemHeightProperty =
            DependencyProperty.Register("ItemHeight", typeof(double), typeof(FillSizeWrapPanel), new PropertyMetadata(double.NaN, OnItemHeightChanged));

        private static void OnItemHeightChanged(object sender, DependencyPropertyChangedEventArgs e) {
            ((FillSizeWrapPanel)sender).OnItemHeightChanged((double)e.NewValue, (double)e.OldValue);
        }

        private void OnItemHeightChanged(double newValue, double oldValue) {
            this.InvalidateMeasure();
        }

        protected override Size MeasureOverride(Size availableSize) {

            double itemMinWidth = double.IsNaN(this.ItemMinWidth) ? availableSize.Width : this.ItemMinWidth;
            double itemHeight = this.ItemHeight;
            bool hasFixedHeight = !double.IsNaN(this.ItemHeight);

            Size totalSize = new Size();
            totalSize.Width = availableSize.Width;

            int columnCount = (int)(availableSize.Width / itemMinWidth);
            columnCount = columnCount == 0 ? 1 : columnCount;
            double itemWidth = availableSize.Width / columnCount;

            int childCount = 1;
            double lineMaxHeight = 0;
            foreach (UIElement child in this.Children) {

                child.Measure(new Size(hasFixedHeight ? itemHeight : availableSize.Height,
                                       itemWidth));

                Size desiredSize = child.DesiredSize;

                double height = hasFixedHeight ? itemHeight : desiredSize.Height;

                lineMaxHeight = Math.Max(lineMaxHeight, height);
                if ((childCount % columnCount) == 0) {
                    totalSize.Height += lineMaxHeight;
                    lineMaxHeight = 0.0;
                }

                childCount++;
            }

            if (((childCount - 1) % columnCount) != 0) {
                totalSize.Height += lineMaxHeight;
            }

            return totalSize;
        }

        protected override Size ArrangeOverride(Size finalSize) {
            double itemMinWidth = double.IsNaN(this.ItemMinWidth) ? finalSize.Width : this.ItemMinWidth;
            double itemHeight = this.ItemHeight;
            bool hasFixedHeight = !double.IsNaN(this.ItemHeight);

            Size totalSize = new Size();
            totalSize.Width = finalSize.Width;

            int columnCount = (int)(finalSize.Width / itemMinWidth);
            columnCount = columnCount == 0 ? 1 : columnCount;
            double itemWidth = finalSize.Width / columnCount;

            int childCount = 1;
            double lineMaxHeight = 0;
            foreach (UIElement child in this.Children) {
                Size desiredSize = child.DesiredSize;

                double height = hasFixedHeight ? itemHeight : desiredSize.Height;

                double x = childCount == 1 ? 0 : ((childCount - 1) % columnCount) * itemWidth;
                child.Arrange(new Rect(x, totalSize.Height, itemWidth, height));

                lineMaxHeight = Math.Max(lineMaxHeight, height);
                if ((childCount % columnCount) == 0) {
                    totalSize.Height += lineMaxHeight;
                    lineMaxHeight = 0.0;
                }

                childCount++;
            }

            return finalSize;
        }
    }
}
