﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Markup;
using System.Windows.Controls.Primitives;

namespace Sample.Controls {

    [ContentProperty("Pages")]
    public class PagingContentControl : Control {

        private ButtonBase previouseButton = null;
        private ButtonBase nextButton = null;
        private ContentControl contentHolder = null;

        public DependencyObjectCollection<UIElement> Pages {
            get { return (DependencyObjectCollection<UIElement>)GetValue(PagesProperty); }
            set { SetValue(PagesProperty, value); }
        }

        public static readonly DependencyProperty PagesProperty =
            DependencyProperty.Register("Pages", typeof(DependencyObjectCollection<UIElement>), typeof(PagingContentControl), new PropertyMetadata(OnPagesChanged));

        private static void OnPagesChanged(object sender, DependencyPropertyChangedEventArgs e) {
            PagingContentControl control = sender as PagingContentControl;
            if (control != null) {
                control.OnPageChanged(e.NewValue as DependencyObjectCollection<UIElement>,
                                      e.OldValue as DependencyObjectCollection<UIElement>);
            }
        }

        private void OnPageChanged(DependencyObjectCollection<UIElement> newValue, DependencyObjectCollection<UIElement> oldValue) {
            if (newValue != null) {
                newValue.CollectionChanged += PagesCollectionChanged;
            }
            if (oldValue != null) {
                newValue.CollectionChanged -= PagesCollectionChanged;
            }
        }

        private void PagesCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e) {
            if (templateApplied) {
                UpdateDisplay();
            }
        }

        public int CurrentIndex {
            get { return (int)GetValue(CurrentIndexProperty); }
            set { SetValue(CurrentIndexProperty, value); }
        }

        public static readonly DependencyProperty CurrentIndexProperty =
            DependencyProperty.Register("CurrentIndex", typeof(int), typeof(PagingContentControl), new PropertyMetadata(0, OnCurrentIndexChanged));

        private static void OnCurrentIndexChanged(object sender, DependencyPropertyChangedEventArgs e) {
            PagingContentControl control = sender as PagingContentControl;
            if (control != null) {
                control.OnCurrentIndexChanged((int)e.NewValue, (int)e.OldValue);
            }
        }

        private void OnCurrentIndexChanged(int newValue, int oldValue) {
            if (templateApplied) {
                UpdateDisplay();
            }
        }


        public PagingContentControl() {
            
            this.DefaultStyleKey = typeof(PagingContentControl);

            this.Pages = new DependencyObjectCollection<UIElement>();
        }

        private bool templateApplied = false;
        private bool suspendUpdate = false;

        public override void OnApplyTemplate() {
            if (templateApplied) {
                return;
            }
            base.OnApplyTemplate();

            this.previouseButton = this.GetTemplateChild("PreviousButton") as ButtonBase;
            this.nextButton = this.GetTemplateChild("NextButton") as ButtonBase;
            this.contentHolder = this.GetTemplateChild("ContentHolder") as ContentControl;

            if (this.previouseButton != null) {
                this.previouseButton.Click += PreviouseButton_Click;
            }
            if (this.nextButton != null) {
                this.nextButton.Click += NextButton_Click;
            }

            templateApplied = true;
            this.UpdateDisplay();
        }

        private void PreviouseButton_Click(object sender, RoutedEventArgs e) {
            this.CurrentIndex = this.CurrentIndex - 1;
        }

        private void NextButton_Click(object sender, RoutedEventArgs e) {
            this.CurrentIndex = this.CurrentIndex + 1;
        }

        private void UpdateDisplay() {
            if (!templateApplied || suspendUpdate) {
                return;
            }
            if (this.contentHolder == null) {
                return;
            }

            if (this.Pages == null || this.Pages.Count == 0) {
                this.contentHolder.ClearValue(ContentControl.ContentProperty);
                return;
            }
            
            int targetIndex = Math.Min(this.Pages.Count - 1, Math.Max(this.CurrentIndex, 0));
            if (targetIndex != this.CurrentIndex) {
                suspendUpdate = true;
                this.CurrentIndex = targetIndex;
                suspendUpdate = false;
            }
            Object currentContent = this.contentHolder.Content;
            UIElement newContent = this.Pages[targetIndex];
            if (currentContent != newContent) {
                this.contentHolder.Content = this.Pages[targetIndex];
            }
            if(this.previouseButton != null) {
                if (targetIndex == 0) {
                    this.previouseButton.Visibility = Visibility.Collapsed;
                } else {
                    this.previouseButton.Visibility = Visibility.Visible;
                }
            }
            if (this.nextButton != null) {
                if (targetIndex >= this.Pages.Count - 1) {
                    this.nextButton.Visibility = Visibility.Collapsed;
                } else {
                    this.nextButton.Visibility = Visibility.Visible;
                }
            }
            
        }
    }
}
