﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interactivity;
using System.Windows.Controls.Primitives;
using System.ComponentModel;
using System.Windows.Data;
using System.Collections.Generic;

namespace Sample.Interactivity {

    [DefaultTrigger(typeof(UIElement), typeof(System.Windows.Interactivity.EventTrigger), "MouseLeftButtonDown")]
    [DefaultTrigger(typeof(ButtonBase), typeof(System.Windows.Interactivity.EventTrigger), "Click")]
    [DefaultTrigger(typeof(Selector), typeof(System.Windows.Interactivity.EventTrigger), "SelectionChanged")]
    public class CollectionViewSourceAction : TargetedTriggerAction<CollectionViewSource> {

        public CollectionViewSourceAction() {
            this.SortDescriptions = new SortDescriptionCollection();
        }

        public SortDescriptionCollection SortDescriptions {
            get { return (SortDescriptionCollection)GetValue(SortDescriptionsProperty); }
            set { SetValue(SortDescriptionsProperty, value); }
        }

        public static readonly DependencyProperty SortDescriptionsProperty =
            DependencyProperty.Register("SortDescriptions", typeof(SortDescriptionCollection),
            typeof(CollectionViewSourceAction), new PropertyMetadata(null));


        public IEnumerable<GroupDescription> GroupDescriptions {
            get { return (IEnumerable<GroupDescription>)GetValue(GroupDescriptionsProperty); }
            set { SetValue(GroupDescriptionsProperty, value); }
        }

        public static readonly DependencyProperty GroupDescriptionsProperty =
            DependencyProperty.Register("GroupDescriptions", typeof(IEnumerable<GroupDescription>),
            typeof(CollectionViewSourceAction), new PropertyMetadata(null));

        protected override void Invoke(object parameter) {
            if (this.Target == null) {
                return;
            }

            using (this.Target.DeferRefresh()) {

                if (this.SortDescriptions != null && this.SortDescriptions.Count > 0) {
                    this.Target.SortDescriptions.Clear();
                    foreach (SortDescription sort in this.SortDescriptions) {
                        this.Target.SortDescriptions.Add(sort);
                    }
                }

                if (this.GroupDescriptions != null) {
                    this.Target.GroupDescriptions.Clear();
                    foreach (GroupDescription groupDescription in this.GroupDescriptions) {
                        this.Target.GroupDescriptions.Add(groupDescription);
                    }
                }
            }
        }
    }
}
