﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Expression.Controls;
using Microsoft.Expression.Media;
using Microsoft.Expression.Shapes;

namespace Sample.Shapes {

    public class HomebaseShape : PrimitiveShape, IHomebaseShapeGeometrySourceParameters {

        public static readonly DependencyProperty RadiusProperty =
           DependencyProperty.Register("Radius", typeof(double), typeof(HomebaseShape), new PropertyMetadata(10d, OnRadiusPropertyChanged));

        private static void OnRadiusPropertyChanged(object sender, DependencyPropertyChangedEventArgs e) {
            HomebaseShape shape = sender as HomebaseShape;
            if (shape != null && e.NewValue != e.OldValue) {
                shape.InvalidateGeometry(InvalidateGeometryReasons.PropertyChanged);
            }
        }

        public double Radius {
            get { return (double)GetValue(RadiusProperty); }
            set { SetValue(RadiusProperty, value); }
        }

        public static readonly DependencyProperty HeadAngleProperty =
            DependencyProperty.Register("HeadAngle", typeof(double), typeof(HomebaseShape), new PropertyMetadata(90d, OnHeadAnglePropertyChanged));

        private static void OnHeadAnglePropertyChanged(object sender, DependencyPropertyChangedEventArgs e) {
            HomebaseShape shape = sender as HomebaseShape;
            if (shape != null && e.NewValue != e.OldValue) {
                shape.InvalidateGeometry(InvalidateGeometryReasons.PropertyChanged);
            }
        }

        public double HeadAngle {
            get { return (double)GetValue(HeadAngleProperty); }
            set { SetValue(HeadAngleProperty, value); }
        }

        public static readonly DependencyProperty OrientationProperty =
            DependencyProperty.Register("Orientation", typeof(HomebaseOrientation), typeof(HomebaseShape), new PropertyMetadata(HomebaseOrientation.Right, OnOrientationChanged));

        private static void OnOrientationChanged(object sender, DependencyPropertyChangedEventArgs e) {
            HomebaseShape shape = sender as HomebaseShape;
            if (shape != null && e.NewValue != e.OldValue) {
                shape.InvalidateGeometry(InvalidateGeometryReasons.PropertyChanged);
            }
        }

        public HomebaseOrientation Orientation {
            get { return (HomebaseOrientation)GetValue(OrientationProperty); }
            set { SetValue(OrientationProperty, value); }
        }

        protected override IGeometrySource CreateGeometrySource() {
            return new HomebaseShapeGeometrySource();
        }
    }
}
