﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Expression.Media;
using Microsoft.Expression.Drawing.Core;

namespace Sample.Shapes {

    public class HomebaseShapeGeometrySource : GeometrySource<IHomebaseShapeGeometrySourceParameters> {

        public HomebaseShapeGeometrySource() {

        }

        protected override bool UpdateCachedGeometry(IHomebaseShapeGeometrySourceParameters parameters) {
            double height = this.LayoutBounds.Height;
            double width = this.LayoutBounds.Width;
            double radius = Math.Max(0, Math.Min(parameters.Radius, (Math.Min(height, width) / 2d)));
            double headAngle = Math.Max(0, Math.Min(180d, parameters.HeadAngle));

            PathGeometry geometry = null;
            if (parameters.Orientation == HomebaseOrientation.Right) {
                geometry = CreateRightHeadGeometory(height, width, radius, headAngle);
            } else if (parameters.Orientation == HomebaseOrientation.Left) {
                geometry = CreateLeftHeadGeometory(height, width, radius, headAngle);
            }

            this.cachedGeometry = geometry;
            return true;
        }

        private PathGeometry CreateRightHeadGeometory(double height, double width, double radius, double headAngle) {

            PathGeometry geometry = new PathGeometry();
            PathFigure figure = new PathFigure() { IsClosed = true };
            double halfRadius = radius / 2d;

            double headWidth = (height / 2d) * Math.Tan((90d - (headAngle / 2d)).ToRadian());
            if (headWidth > width - radius) {
                headWidth = width - radius;
                headAngle = Math.Atan((height / 2d) / headWidth);
            }

            //斜辺
            double hypotenuseLength = (height / 2d) / Math.Cos((90d - (headAngle / 2d)).ToRadian());

            figure.StartPoint = new Point(0, radius);
            figure.Segments.Add(new BezierSegment() {
                Point1 = new Point(0, halfRadius),
                Point2 = new Point(halfRadius, 0),
                Point3 = new Point(radius, 0)
            });
            figure.Segments.Add(new LineSegment() { Point = new Point(width - headWidth, 0) });
            figure.Segments.Add(new LineSegment() { Point = new Point(width, height / 2d) });
            figure.Segments.Add(new LineSegment() { Point = new Point(width - headWidth, height) });
            figure.Segments.Add(new LineSegment() { Point = new Point(radius, height) });
            figure.Segments.Add(new BezierSegment() {
                Point1 = new Point(halfRadius, height),
                Point2 = new Point(0, height - halfRadius),
                Point3 = new Point(0, height - radius)
            });

            geometry.Figures.Add(figure);

            return geometry;
        }

        private PathGeometry CreateLeftHeadGeometory(double height, double width, double radius, double headAngle) {
            PathGeometry geometry = new PathGeometry();
            PathFigure figure = new PathFigure() { IsClosed = true };
            double halfRadius = radius / 2d;

            double headWidth = (height / 2d) * Math.Tan((90d - (headAngle / 2d)).ToRadian());
            if (headWidth > width - radius) {
                headWidth = width - radius;
                headAngle = Math.Atan((height / 2d) / headWidth);
            }

            //斜辺
            double hypotenuseLength = (height / 2d) / Math.Cos((90d - (headAngle / 2d)).ToRadian());

            figure.StartPoint = new Point(width, radius);
            figure.Segments.Add(new BezierSegment() {
                Point1 = new Point(width, halfRadius),
                Point2 = new Point(width - halfRadius, 0),
                Point3 = new Point(width - radius, 0)
            });
            figure.Segments.Add(new LineSegment() { Point = new Point(headWidth, 0) });
            figure.Segments.Add(new LineSegment() { Point = new Point(0, height / 2d) });
            figure.Segments.Add(new LineSegment() { Point = new Point(headWidth, height) });
            figure.Segments.Add(new LineSegment() { Point = new Point(width - radius, height) });
            figure.Segments.Add(new BezierSegment() {
                Point1 = new Point(width - halfRadius, height),
                Point2 = new Point(width, height - halfRadius),
                Point3 = new Point(width, height - radius)
            });

            geometry.Figures.Add(figure);

            return geometry;

        }
    }

	static class Extensions {

		public static double ToRadian(this double degree) {
			return (Math.PI / 180d) * degree;
		}

		public static double ToDegree(this double radian) {
			return radian * (180d / Math.PI);
		}

	}
}
